<?php

function get_nonce_key() {
	/* 「my_nonce」の部分をランダムな英数字に変更する */
	$nonce_key = 'my_nonce';
	return $nonce_key;

}

if ( !is_admin() ) {

	function tsearch_register_style() {

		wp_register_style(
			'taxonomy-search-css',
			tsearch_css . '/tsearch.min.css'
		);

	}

	function tsearch_register_script() {

		wp_register_script(
			'taxonomy-search',
			tsearch_js . '/tsearch.min.js',
			array( 'jquery' ),
			'1.0',
			true
		);

	}

	function tsearch_nonce() {

		$nonce_key = get_nonce_key();

		wp_localize_script(
			'taxonomy-search',
			'Search',
			array(
				'url'    => admin_url( 'admin-ajax.php' ),
				'action' => 'taxonomy_search',
				'token'  => wp_create_nonce( $nonce_key )
			)
		);

	}

	function tsearch_add_style() {

		tsearch_register_style();
		wp_enqueue_style( 'taxonomy-search-css' );

	}
	add_action( 'wp_print_styles', 'tsearch_add_style' );

	function tsearch_add_script() {

		tsearch_register_script();
		tsearch_nonce();
		wp_enqueue_script( 'taxonomy-search' );

	}
	add_action( 'wp_enqueue_scripts', 'tsearch_add_script' );

}

function code_taxonomy_search() {

	$cat_args = array(
		'show_count'      => TRUE,
		'echo'            => FALSE,
		'id'              => 'category',
		'name'            => 'cat',
		'orderby'         => 'count',
		'order'           => 'DESC',
		'taxonomy'        => 'category',
		'show_option_all' => 'すべてのカテゴリ'
	);

	$tag_args = array(
		'orderby'         => 'count',
		'order'           => 'DESC'
	);

	$tags = get_tags( $tag_args );

	$form  = "<div id='search-box' style='margin-bottom: 28px;'>\n";
	$form .= "<form method='GET' action='" . home_url( '/' ) . "'>\n";

	$form .= wp_dropdown_categories( $cat_args ) . "\n";

	$form .= "<select id='post_tag' name='tag'>\n";
	$form .= "<option value=''>すべてのタグ</option>\n";
	foreach ( $tags as $value ) {

		$opt_val = esc_attr( $value->slug );
		$opt_str = esc_attr( $value->name . " (" . $value->count . ")" );

		$form .= "<option value='$opt_val'>$opt_str</option>\n";

	}

	$form .= "</select>\n";

	$form .= "<p><input id='submit' type='submit' value='検索' /></p>\n";
	$form .= "</form>\n";
	$form .= "</div>\n";

	return $form;

}
add_shortcode( 'taxonomy-search', 'code_taxonomy_search' );
add_filter('widget_text', 'do_shortcode');

function ajax_taxonomy_search() {

	$json = array(
		'status' => 'NG',
		'term'   => array()
	);

	$nonce_key = get_nonce_key();

	header( 'Content-Type: application/json; charset=UTF-8' );

	if ( wp_verify_nonce( $_POST['token'], $nonce_key ) && isset( $_POST['term_id'] ) ) {

		if ( $_POST['term_id'] === 'all' ) {

			$tag_args = array(
				'orderby' => 'count',
				'order'   => 'DESC'
			);

			$tags = get_tags( $tag_args );

			foreach ( $tags as $value ) {

				if ( ! isset( $json['taxonomy'] ) ) { 
					$json['taxonomy'] = $value->taxonomy;
				}

				$term_info = array(
					'id'    => $value->term_id,
					'name'  => $value->name,
					'slug'  => $value->slug,
					'count' => $value->count
				);

				array_push( $json['term'], $term_info );
			}


		} else {

			$json = get_relation_taxonomy( $_POST['term_id'] );

		}

		if ( $json['taxonomy'] ) {

			$json['status'] = 'OK';

		}

	}

	echo json_encode( $json );

	die();

}
add_action( 'wp_ajax_taxonomy_search', 'ajax_taxonomy_search' );
add_action( 'wp_ajax_nopriv_taxonomy_search', 'ajax_taxonomy_search' );

function get_relation_taxonomy( $term_id = FALSE ) {

	$term_ids = array(
		'term'     => array(),
		'taxonomy' => FALSE
	);

	if ( $term_id === FALSE ) {
		return $term_ids;
	}

	$taxonomy = get_terms(
			array(
				'category',
				'post_tag'
			),
			array(
				'hide_empty' => TRUE,
				'include'    => $term_id
			)
		    );

	$taxonomy = $taxonomy[0]->taxonomy;

	if ( ! is_null( $taxonomy ) ) {

		$args = array(
			'nopaging'            => 1,
			'ignore_sticky_posts' => 1,
			'order'               => ASC,
			'tax_query'           => array( 
				array(
					'taxonomy' => $taxonomy,
					'terms'    => $term_id,
					'field'    => 'id',
					'operator' => 'AND'
				 )
			)
		);

		$my_query = new WP_Query( $args );

		if( $my_query->have_posts() ) {


			if ( $taxonomy === 'category' ) {
				$taxonomy = 'post_tag';
			} else {
				$taxonomy = 'category';
			}

			while ( $my_query->have_posts() ) : $my_query->the_post();

				$term = get_the_terms( get_the_ID(), $taxonomy );

				if ( $term ) {

					foreach ( $term as $value ) {

						if ( ! isset( $term_ids['term'][$value->term_id] ) ) {

							$term_info = array(
								'id'   => $value->term_id,
								'name' => $value->name,
								'slug' => $value->slug,
							);

							$term_ids['term'][$value->term_id] = $term_info;

						}

						$term_ids['term'][$value->term_id]['count'] += 1;

					}

				}

			endwhile;

			wp_reset_query();

		}

		if ( $term_ids['term'] ) {

			$term_ids['term'] = array_merge( $term_ids['term'] );

			foreach ( $term_ids['term'] as $key => $value ) {
				$key_id[$key] = $value['count'];
			}
			array_multisort( $key_id, SORT_DESC, $term_ids['term'] );

			$term_ids['taxonomy'] = $taxonomy;

		}

	}

	return $term_ids;

}

?>
